@@ -8,15 +8,24 @@
 #define WIN
 #endif
 
-#include <stdlib.h>
 #ifdef WIN
 #include <windows.h>
 #else
 #ifndef NOTHREADS
 #include <pthread.h>
+
+#ifdef _POSIX_BARRIERS
+#define COOP
+#endif
+
+/* It's slower, too much synchronization */
+#undef COOP
+
 #endif
 #endif
 
+#include <stdlib.h>
+#include <time.h>
 #include <math.h>
 #include <string.h>
 #include <R.h>
@@ -34,8 +43,12 @@ typedef struct {
   int *ii;  /* indices into gpl */
 } FACTOR;
 
+#if __GNUC__ > 3 && __GNUC_MINOR__ > 3
+/*#pragma GCC optimize ("O3", "loop-block", "unroll-loops")*/
+/*#pragma GCC optimize ("O3", "unroll-loops")*/
+#endif
 
-static double gmean(double *v,int N, FACTOR *f,double *means) {
+static R_INLINE double gmean(double *v,int start, int end, FACTOR *f,double *means) {
   /*
     Do the actual work, single vector, single factor, 
     output in preallocated vector means
@@ -50,12 +63,11 @@ static double gmean(double *v,int N, FACTOR *f,double *means) {
 
   /* add entries into right group */
 
-  for(i = 0; i < N; i++) {
+  for(i = start; i < end; i++) {
     means[group[i]-1] += v[i];
   }
 
   /* divide by group size */
-
   for(i = 0; i < nlev; i++) {
     means[i] *= f->invgpsize[i];
     sum += means[i]*means[i];
@@ -67,7 +79,7 @@ static double gmean(double *v,int N, FACTOR *f,double *means) {
   Centre on all factors in succession.  Vector v. In place.
  */
 
-static double centre(double *v, int N, 
+static R_INLINE double centre(double *v, int N, 
 		   FACTOR *factors[], int e, double *means) {
 
   int i;
@@ -78,7 +90,7 @@ static double centre(double *v, int N,
     const int *gp = f->group;
     int j=0;
     /* compute means */
-    sum += gmean(v,N,f,means);
+    sum += gmean(v,0,N,f,means);
 
     for(j = 0; j < N; j++) {
       v[j] -= means[gp[j]-1];
@@ -90,7 +102,8 @@ static double centre(double *v, int N,
 
 /*
   Method of alternating projections.  Input v, output res.
- */
+*/
+
 static int demean(double *v, int N, double *res,
 		   FACTOR *factors[],int e,double eps, double *means) {
   double delta;
@@ -102,6 +115,7 @@ static int demean(double *v, int N, double *res,
   int iter;
   int okconv = 1;
   /* make a copy to result vector, centre() is in-place */
+
   memcpy(res,v,N*sizeof(double));
 
   if(1 == e) {
@@ -146,6 +160,17 @@ typedef struct {
   double eps;
   int nowdoing;
   int badconv;
+  time_t last;
+  time_t start;
+  int done;
+#ifdef COOP
+  double **means;
+  double sum;
+  int threads;
+  int tmp;
+  double *gmeans;
+  pthread_barrier_t *barrier;
+#endif
 #ifndef NOTHREADS
 #ifdef WIN
   HANDLE *lock;
@@ -163,6 +188,12 @@ typedef struct {
   by a mutex.
  */
 
+#ifdef COOP
+#define BARRIER(b) pthread_barrier_wait(b)
+#else
+#define BARRIER(b)
+#endif
+
 #ifdef NOTHREADS
 #define LOCK(l)
 #define UNLOCK(l)
@@ -177,37 +208,196 @@ typedef struct {
 #endif
 
 
-/* The thread routine */
+/* Cooperative centering 
+   Each vector is centered by all the threads.
+   Each thread gets a portion of the vector.
+   This is faster with few vectors, but there's 
+   synchronization which may cause it to slow down.
+   Indeed it's slow.  Deactivated.
+
+   Haven't bothered to figure out how to do barriers in windows yet,
+   so skip the cooperation threading there.
+*/
+
+#ifdef COOP
+
 #ifdef WIN
-DWORD WINAPI demeanlist_thr(LPVOID varg) {
+DWORD WINAPI demeanlist_coop(LPVOID varg) {
 #else
-static void *demeanlist_thr(void *varg) {
+static void *demeanlist_coop(void *varg) {
+#endif
+  PTARG *arg = (PTARG*) varg;
+  int maxlev;
+  int i;
+  int myid;
+  double *means;
+  int N = arg->N;
+  int e = arg->e;
+  int vecstart,veclen,vecend;
+  /* preallocate means */
+  maxlev = 0;
+  for(i = 0; i < e; i++)
+    if(maxlev < arg->factors[i]->nlevels) maxlev = arg->factors[i]->nlevels;
+
+  /* Figure out which slice of the vectors I'm going to compute on */
+  /* First, number the threads and store the means pointer */
+
+  LOCK(arg->lock);
+  myid = (arg->tmp)++;
+  means = Calloc(maxlev,double);
+  arg->means[myid] = means;
+  UNLOCK(arg->lock);
+
+  BARRIER(arg->barrier);
+  veclen = N / arg->threads;
+  vecstart = veclen*myid;
+  /* Stuff the rest into the last */
+  if(myid == arg->threads-1) {
+    veclen = N - vecstart;
+  }
+
+  vecend = vecstart + veclen;
+  for(i = 0; i < arg->K; i++) {
+    time_t start = time(NULL);
+    double sd;
+    double c;
+    double prevdiff;
+    double neps;
+    double *v = arg->v[i];
+    double *t = arg->res[i];
+    int k;
+    /* Centre this vector */
+    /* copy to target */
+    neps = 0;
+    for(k = 0; k < N; k++) {
+      neps += v[k]*v[k];
+    }
+    prevdiff = 2.0*sqrt(neps);
+    neps = arg->eps*sqrt(neps);
+    if(myid == 0) {
+      memcpy(t,v,N*sizeof(double));	
+      arg->sum = 0;
+    }
+    /* All wait for the copy to the common area */
+    BARRIER(arg->barrier);
+    
+    do {
+      int j;
+      /* Compute the means */
+      /* Zero the square sums */
+      if(BARRIER(arg->barrier) == PTHREAD_BARRIER_SERIAL_THREAD) {
+	arg->sum = 0;
+      }
+      BARRIER(arg->barrier);
+
+      for(j = 0; j < e; j++) {
+	FACTOR *f = arg->factors[j];
+	int *gp = f->group;
+	int idx;
+
+	gmean(t,vecstart,vecend,f,means);
+	/* All wait for the means */
+	if(BARRIER(arg->barrier) == PTHREAD_BARRIER_SERIAL_THREAD) {
+	  int k,l;
+	  double sum = 0;
+	  /* sum the means from all threads, we're in only one now */
+	  memset(arg->gmeans,0, f->nlevels*sizeof(double));
+	  for(l = 0; l < arg->threads; l++) {
+	    for(k = 0; k < f->nlevels; k++) {
+	      arg->gmeans[k] += arg->means[l][k];
+	    }
+	  }
+	  /* add to the square sum */
+	  for(k = 0; k < f->nlevels; k++) {
+	    sum += arg->gmeans[k] * arg->gmeans[k];
+	  }
+	  /* store it */
+	  arg->sum += sum;
+	}
+	/* all wait for the means */
+	BARRIER(arg->barrier);
+	/* subtract the means, own slice */
+	for(idx = vecstart; idx < vecend; idx++) {
+	  t[idx] -= arg->gmeans[gp[idx]-1];
+	}
+      }
+      BARRIER(arg->barrier);
+      sd = sqrt(arg->sum);
+      c = sd/prevdiff;
+      if(c >= 1.0) {
+	if(myid == 0) {
+	  arg->badconv++;
+	}
+	break;
+      }
+      prevdiff = sd;
+    } while(sd > neps*(1.0-c));
+    now = time(NULL);
+    arg->done++;
+    if(now > arg->last + 300) {
+      Rprintf("...finished centering vector %d of %d in %d(%d) seconds\n",
+	      i+1,arg->K,now-start,now-arg->start);
+      arg->last = now;
+    }
+  }
+  LOCK(arg->lock);
+  Free(means);
+  UNLOCK(arg->lock);
+  return 0;
+}
+#define demeanlist_thr demeanlist_coop
+#else
+#define demeanlist_thr demeanlist_single
+#endif
+
+
+/* The thread routine 
+   Each thread centres an entire vector.
+*/
+#ifdef WIN
+DWORD WINAPI demeanlist_single(LPVOID varg) {
+#else
+static void *demeanlist_single(void *varg) {
 #endif
   PTARG *arg = (PTARG*) varg;
   double *means;
   int i;
-  int maxlev;
+  int maxlev=0;
   int vecnum;
   int okconv = 0;
+
   /* preallocate means */
-  maxlev = 0;
   for(i = 0; i < arg->e; i++)
     if(maxlev < arg->factors[i]->nlevels) maxlev = arg->factors[i]->nlevels;
+  LOCK(arg->lock);  /* In case Calloc is not thread-safe */
   means = Calloc(maxlev,double);
+  UNLOCK(arg->lock);
   while(1) {
+    time_t start,now;
+    /* Find next vector */
     LOCK(arg->lock);
     vecnum = (arg->nowdoing)++;
     UNLOCK(arg->lock);
     if(vecnum >= arg->K) break;
-    okconv = demean(arg->v[vecnum],arg->N,arg->res[vecnum],arg->factors,arg->e,arg->eps,means);
-    if(!okconv) {
-      LOCK(arg->lock);
-      arg->badconv++;
-      UNLOCK(arg->lock);
-    }
-  }
 
+    start = time(NULL);
+    okconv = demean(arg->v[vecnum],arg->N,arg->res[vecnum],arg->factors,arg->e,arg->eps,means);
+    now = time(NULL);
+    LOCK(arg->lock);
+    if(!okconv) {
+      arg->badconv++;
+    }
+    (arg->done)++;
+    if(now > arg->last + 300) {
+      Rprintf("...finished centering vector %d(%d) of %d in %d(%d) seconds\n",
+	      arg->done,vecnum+1,arg->K,now-arg->start,now-start);
+      arg->last = now;
+    }
+    UNLOCK(arg->lock);
+  }
+  LOCK(arg->lock);
   Free(means);
+  UNLOCK(arg->lock);
   return 0;
 }
 
@@ -216,11 +406,11 @@ static void *demeanlist_thr(void *varg) {
 static int demeanlist(double **vp, int N, int K, double **res,
 		       FACTOR *factors[], int e, double eps,int cores) {
   PTARG arg;
-
 #ifndef NOTHREADS
+  int numthr = 1;
+  int i;
   int thr;
-  int numthr;
-
+  int maxlev;
 #ifdef WIN
   HANDLE *threads;
   DWORD *threadids;
@@ -229,10 +419,18 @@ static int demeanlist(double **vp, int N, int K, double **res,
   pthread_t *threads;
   pthread_mutex_t lock = PTHREAD_MUTEX_INITIALIZER;
 #endif
+#ifdef COOP
+  pthread_barrier_t barrier;
+#endif
+
+  maxlev = 0;
+  for(i = 0; i < e; i++)
+    if(maxlev < factors[i]->nlevels) maxlev = factors[i]->nlevels;
+
 
   numthr = cores;
   if(numthr > K) numthr = K;
-
+  if(numthr < 1) numthr = 1;
 #ifdef WIN
   lock = CreateMutex(NULL,FALSE,NULL);
   threads = (HANDLE*) R_alloc(numthr,sizeof(HANDLE));
@@ -242,6 +440,10 @@ static int demeanlist(double **vp, int N, int K, double **res,
 #endif
 
   arg.lock = &lock;
+#ifdef COOP
+  arg.barrier = &barrier;
+  pthread_barrier_init(arg.barrier,NULL,numthr);
+#endif
 #endif
 
   arg.badconv = 0;
@@ -251,8 +453,17 @@ static int demeanlist(double **vp, int N, int K, double **res,
   arg.K = K;
   arg.nowdoing = 0;
   arg.factors = factors;
+  arg.last = time(NULL);
+  arg.start = arg.last;
+  arg.done = 0;
   arg.e = e;
   arg.eps = eps;
+#ifdef COOP
+  arg.means = (double**) R_alloc(numthr,sizeof(double*));
+  arg.gmeans = (double*) R_alloc(maxlev,sizeof(double));
+  arg.threads = numthr;
+  arg.tmp = 0;
+#endif
 
 #ifdef NOTHREADS
   demeanlist_thr((void*)&arg);
@@ -280,9 +491,12 @@ static int demeanlist(double **vp, int N, int K, double **res,
   for(thr = 0; thr < numthr; thr++) {
     (void)pthread_join(threads[thr], NULL);
   }
+#ifdef COOP
+  pthread_barrier_destroy(arg.barrier);
 #endif
 #endif
-
+#endif
+  if(arg.start != arg.last) Rprintf("...all vectors centred in %d seconds\n",time(NULL)-arg.start);
   return(arg.badconv);
 }
 
@@ -477,51 +691,23 @@ static SEXP R_kaczmarz(SEXP flist, SEXP RR, SEXP Reps) {
 static SEXP R_demeanlist(SEXP vlist, SEXP flist, SEXP Ricpt, SEXP Reps, SEXP scores) {
   int numvec;
   int numfac;
+  int cnt;
   double **vectors;
   double **target;
   double eps;
-  SEXP resvec;
   SEXP reslist;
   FACTOR **factors;
   int cores;
   int i;
-  int N=0, len;
-  int mtinput = 0;
+  int N=0;
   int icpt;
   SEXP badconv;
+  int listlen;
+  
   icpt = INTEGER(Ricpt)[0] - 1; /* convert from 1-based to zero-based */
   eps = REAL(Reps)[0];
   cores = INTEGER(scores)[0];
-  if(!IS_NUMERIC(vlist)) {
-    PROTECT(vlist = AS_LIST(vlist));
-    numvec = LENGTH(vlist);
 
-    vectors = (double **)R_alloc(numvec,sizeof(double*));
-    /* Create the vector pointers */
-    for(i = 0; i < numvec; i++) {
-      len = LENGTH(VECTOR_ELT(vlist,i));
-      if(i > 0 && len != N) {
-	error("Vectors must have equal length");
-      }
-      vectors[i] = REAL(VECTOR_ELT(vlist,i));
-      N = len;
-    }
-  } else {
-    int ncol;
-    mtinput = 1;
-    PROTECT(vlist = vlist);
-    N = nrows(vlist);
-    ncol = ncols(vlist);
-
-    vectors = (double **)R_alloc(ncol,sizeof(double*));
-    /* Create the vector pointers, skip intercept */
-    numvec = 0;
-    for(i = 0; i < ncol; i++) {
-      /* to the i'th column */
-      if(i == icpt) continue;
-      vectors[numvec++] = REAL(vlist) + i*N;
-    }
-  } 
 
   PROTECT(flist = AS_LIST(flist));
   numfac = LENGTH(flist);
@@ -529,10 +715,13 @@ static SEXP R_demeanlist(SEXP vlist, SEXP flist, SEXP Ricpt, SEXP Reps, SEXP sco
   factors = (FACTOR**) R_alloc(numfac,sizeof(FACTOR*));
   for(i = 0; i < numfac; i++) {
     int j;
+    int len;
     FACTOR *f;
     len = LENGTH(VECTOR_ELT(flist,i));
-    if(len != N) {
-       error("Factors and vectors must have the same length %d %d",len,N);
+    if(i == 0) {
+      N = len;
+    } else if(len != N) {
+      error("All factors must have the same length %d %d",len,N);
     }
 
     factors[i] = (FACTOR*) R_alloc(1,sizeof(FACTOR));
@@ -549,8 +738,11 @@ static SEXP R_demeanlist(SEXP vlist, SEXP flist, SEXP Ricpt, SEXP Reps, SEXP sco
     for(j = 0; j < N; j++) {
       /* skip entries without a group, do we need that? */
       /* if(f->group[j] < 1) error("Factors can't have missing levels"); */
-      if(f->group[j] > 0)
+      if(f->group[j] > 0) {
 	f->invgpsize[f->group[j]-1] += 1.0;
+      } else {
+	error("Factors can't have missing levels");
+      }
     }
     /* then invert it, it's much faster to multiply than to divide */
     /* in the iterations */
@@ -560,29 +752,74 @@ static SEXP R_demeanlist(SEXP vlist, SEXP flist, SEXP Ricpt, SEXP Reps, SEXP sco
     /*invertfactor(f,N);*/
   }
 
-  /* allocate result structure */
 
+  /* Then the vectors */
+
+  PROTECT(vlist = AS_LIST(vlist));
+  listlen = LENGTH(vlist);
+  PROTECT(reslist = NEW_LIST(listlen));
+
+  /* First, count the number of vectors in total */
+  numvec = 0;
+  for(i = 0; i < listlen; i++) {
+    SEXP elt = VECTOR_ELT(vlist,i);
+    if(!IS_NUMERIC(elt)) error("Entries must be numeric");
+    /* Each entry in the list is either a vector or a matrix */
+    if(!isMatrix(elt)) {
+      if(LENGTH(elt) != N) 
+	error("Factors and vectors must have the same length %d != %d",N,LENGTH(elt));
+      numvec++;
+    } else {
+      if(nrows(elt) != N)
+	error("Factors and vectors must have the same length %d != %d",N,nrows(elt));
+      numvec += ncols(elt);
+      if(icpt >= 0 && icpt < ncols(elt)) numvec--;
+    }
+  }
+
+  /* Allocate pointers to source vectors */
+  vectors = (double **)R_alloc(numvec,sizeof(double*));
+  /* Allocate pointers to result vectors */
   target = (double**) R_alloc(numvec,sizeof(double*));
-  if(mtinput) {
-    double *mat;
-    PROTECT(reslist = allocMatrix(REALSXP,N,numvec));
-    mat = REAL(reslist);
-    for(i = 0; i < numvec; i++) {
-      target[i] = mat + i*N;
-    }
-  } else {
-    PROTECT(reslist = NEW_LIST(numvec));
-
-    for(i = 0; i < numvec; i++) {
-      PROTECT(resvec = allocVector(REALSXP,N));  /* need protect? */
-      target[i] = REAL(resvec);
+  /* Loop through list again to set up result structure */
+  cnt = 0;
+  for(i = 0; i < listlen; i++) {
+    SEXP elt = VECTOR_ELT(vlist,i);
+    if(!isMatrix(elt)) {
+      /* It's a vector */
+      SEXP resvec;
+      vectors[cnt] = REAL(elt);
+      PROTECT(resvec = allocVector(REALSXP,LENGTH(elt)));
+      target[cnt] = REAL(resvec);
       SET_VECTOR_ELT(reslist,i,resvec);
-      /* as member of list it's protected */
       UNPROTECT(1);
+      cnt++;
+    } else {
+      /* It's a matrix */
+      int cols = ncols(elt);
+      int rcols = cols;
+      int j;
+      SEXP mtx;
+      /* Allocate a matrix */
+      if(icpt >= 0 && icpt < cols) rcols--;
+      PROTECT(mtx = allocMatrix(REALSXP,N,rcols));
+      SET_VECTOR_ELT(reslist,i,mtx);
+      UNPROTECT(1);
+      /* Set up pointers */
+      rcols = 0;
+      for(j = 0; j < cols; j++) {
+	if(j == icpt) continue;
+	vectors[cnt] = REAL(elt) + j*N;
+	target[cnt] = REAL(mtx) + (rcols++)*N;
+	cnt++;
+      }
     }
-  }  
+  }
+
+  /* Then do stuff */
   PROTECT(badconv = allocVector(INTSXP,1));
   INTEGER(badconv)[0] = demeanlist(vectors,N,numvec,target,factors,numfac,eps,cores);
+
   if(INTEGER(badconv)[0] > 0)
     warning("%d vectors failed to centre to tolerance %.1le",INTEGER(badconv)[0],eps);
   setAttrib(reslist,install("badconv"),badconv);
